/*
 * Copyright (c) 2014-2016 ISPRAS (http://www.ispras.ru)
 * Institute for System Programming of the Russian Academy of Sciences
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * ee the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <linux/errno.h>
#include <linux/types.h>
#include <linux/ldv/common.h>
#include <verifier/common.h>
#include <verifier/nondet.h>
#include <verifier/set.h>

struct mutex;

ldv_set LDV_MUTEXES_bl_curve_mutex_of_fb_info;

/* MODEL_FUNC Check that mutex "bl_curve_mutex_of_fb_info" was not locked and lock it */
void ldv_mutex_lock_bl_curve_mutex_of_fb_info(struct mutex *lock)
{
	/* ASSERT Mutex "bl_curve_mutex_of_fb_info" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_bl_curve_mutex_of_fb_info, lock));
	/* NOTE Lock mutex "bl_curve_mutex_of_fb_info" */
	ldv_set_add(LDV_MUTEXES_bl_curve_mutex_of_fb_info, lock);
}

/* MODEL_FUNC Check that mutex "bl_curve_mutex_of_fb_info" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_bl_curve_mutex_of_fb_info(struct mutex *lock)
{
	/* ASSERT Mutex "bl_curve_mutex_of_fb_info" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_bl_curve_mutex_of_fb_info, lock));
	/* NOTE Nondeterministically lock mutex "bl_curve_mutex_of_fb_info" */
	if (ldv_undef_int()) {
		/* NOTE Lock mutex "bl_curve_mutex_of_fb_info" */
		ldv_set_add(LDV_MUTEXES_bl_curve_mutex_of_fb_info, lock);
		/* NOTE Successfully locked mutex "bl_curve_mutex_of_fb_info" */
		return 0;
	}
	else {
		/* NOTE Could not lock mutex "bl_curve_mutex_of_fb_info" */
		return -EINTR;
	}
}

/* MODEL_FUNC Say whether mutex "bl_curve_mutex_of_fb_info" was locked in this or another thread  */
int ldv_mutex_is_locked_bl_curve_mutex_of_fb_info(struct mutex *lock)
{
	/* NOTE Whether mutex "bl_curve_mutex_of_fb_info" was locked in this thread */
	if (ldv_set_contains(LDV_MUTEXES_bl_curve_mutex_of_fb_info, lock)) {
		/* NOTE Mutex "bl_curve_mutex_of_fb_info" was locked in this thread */
		return 1;
	}
	/* NOTE Nondeterministically decide whether mutex "bl_curve_mutex_of_fb_info" was locked in another thread */
	else if (ldv_undef_int()) {
		/* NOTE Mutex "bl_curve_mutex_of_fb_info" was locked in another thread */
		return 1;
	}
	else {
		/* NOTE Mutex "bl_curve_mutex_of_fb_info" was not acquired in this or another thread */
		return 0;
	}
}

/* MODEL_FUNC Lock mutex "bl_curve_mutex_of_fb_info" if it was not locked before */
int ldv_mutex_trylock_bl_curve_mutex_of_fb_info(struct mutex *lock)
{
	/* ASSERT Mutex "bl_curve_mutex_of_fb_info" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock try", !ldv_set_contains(LDV_MUTEXES_bl_curve_mutex_of_fb_info, lock));

	/* NOTE Whether mutex "bl_curve_mutex_of_fb_info" was locked */
	if (ldv_mutex_is_locked_bl_curve_mutex_of_fb_info(lock)) {
		/* NOTE Mutex "bl_curve_mutex_of_fb_info" was locked */
		return 0;
	}
	else {
		/* NOTE Lock mutex "bl_curve_mutex_of_fb_info" */
		ldv_set_add(LDV_MUTEXES_bl_curve_mutex_of_fb_info, lock);
		/* NOTE Successfully locked mutex "bl_curve_mutex_of_fb_info" */
		return 1;
	}
}

/* MODEL_FUNC Decrease counter by one and if it becomes zero check that mutex "bl_curve_mutex_of_fb_info" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_bl_curve_mutex_of_fb_info(atomic_t *cnt, struct mutex *lock)
{
	/* NOTE Decrease counter by one */
	cnt->counter--;

	/* NOTE Whether counter becomes zero */
	if (cnt->counter) {
		/* NOTE Do not lock mutex "bl_curve_mutex_of_fb_info" since counter is greater than zero */
		return 0;
	}
	else {
		ldv_mutex_lock_bl_curve_mutex_of_fb_info(lock);
		/* NOTE Successfully locked mutex "bl_curve_mutex_of_fb_info" */
		return 1;
	}
}

/* MODEL_FUNC Check that mutex "bl_curve_mutex_of_fb_info" was locked and unlock it */
void ldv_mutex_unlock_bl_curve_mutex_of_fb_info(struct mutex *lock)
{
	/* ASSERT Mutex "bl_curve_mutex_of_fb_info" must be locked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double unlock", ldv_set_contains(LDV_MUTEXES_bl_curve_mutex_of_fb_info, lock));
	/* NOTE Unlock mutex "bl_curve_mutex_of_fb_info" */
	ldv_set_remove(LDV_MUTEXES_bl_curve_mutex_of_fb_info, lock);
}
ldv_set LDV_MUTEXES_i_mutex_of_inode;

/* MODEL_FUNC Check that mutex "i_mutex_of_inode" was not locked and lock it */
void ldv_mutex_lock_i_mutex_of_inode(struct mutex *lock)
{
	/* ASSERT Mutex "i_mutex_of_inode" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_i_mutex_of_inode, lock));
	/* NOTE Lock mutex "i_mutex_of_inode" */
	ldv_set_add(LDV_MUTEXES_i_mutex_of_inode, lock);
}

/* MODEL_FUNC Check that mutex "i_mutex_of_inode" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_i_mutex_of_inode(struct mutex *lock)
{
	/* ASSERT Mutex "i_mutex_of_inode" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_i_mutex_of_inode, lock));
	/* NOTE Nondeterministically lock mutex "i_mutex_of_inode" */
	if (ldv_undef_int()) {
		/* NOTE Lock mutex "i_mutex_of_inode" */
		ldv_set_add(LDV_MUTEXES_i_mutex_of_inode, lock);
		/* NOTE Successfully locked mutex "i_mutex_of_inode" */
		return 0;
	}
	else {
		/* NOTE Could not lock mutex "i_mutex_of_inode" */
		return -EINTR;
	}
}

/* MODEL_FUNC Say whether mutex "i_mutex_of_inode" was locked in this or another thread  */
int ldv_mutex_is_locked_i_mutex_of_inode(struct mutex *lock)
{
	/* NOTE Whether mutex "i_mutex_of_inode" was locked in this thread */
	if (ldv_set_contains(LDV_MUTEXES_i_mutex_of_inode, lock)) {
		/* NOTE Mutex "i_mutex_of_inode" was locked in this thread */
		return 1;
	}
	/* NOTE Nondeterministically decide whether mutex "i_mutex_of_inode" was locked in another thread */
	else if (ldv_undef_int()) {
		/* NOTE Mutex "i_mutex_of_inode" was locked in another thread */
		return 1;
	}
	else {
		/* NOTE Mutex "i_mutex_of_inode" was not acquired in this or another thread */
		return 0;
	}
}

/* MODEL_FUNC Lock mutex "i_mutex_of_inode" if it was not locked before */
int ldv_mutex_trylock_i_mutex_of_inode(struct mutex *lock)
{
	/* ASSERT Mutex "i_mutex_of_inode" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock try", !ldv_set_contains(LDV_MUTEXES_i_mutex_of_inode, lock));

	/* NOTE Whether mutex "i_mutex_of_inode" was locked */
	if (ldv_mutex_is_locked_i_mutex_of_inode(lock)) {
		/* NOTE Mutex "i_mutex_of_inode" was locked */
		return 0;
	}
	else {
		/* NOTE Lock mutex "i_mutex_of_inode" */
		ldv_set_add(LDV_MUTEXES_i_mutex_of_inode, lock);
		/* NOTE Successfully locked mutex "i_mutex_of_inode" */
		return 1;
	}
}

/* MODEL_FUNC Decrease counter by one and if it becomes zero check that mutex "i_mutex_of_inode" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_i_mutex_of_inode(atomic_t *cnt, struct mutex *lock)
{
	/* NOTE Decrease counter by one */
	cnt->counter--;

	/* NOTE Whether counter becomes zero */
	if (cnt->counter) {
		/* NOTE Do not lock mutex "i_mutex_of_inode" since counter is greater than zero */
		return 0;
	}
	else {
		ldv_mutex_lock_i_mutex_of_inode(lock);
		/* NOTE Successfully locked mutex "i_mutex_of_inode" */
		return 1;
	}
}

/* MODEL_FUNC Check that mutex "i_mutex_of_inode" was locked and unlock it */
void ldv_mutex_unlock_i_mutex_of_inode(struct mutex *lock)
{
	/* ASSERT Mutex "i_mutex_of_inode" must be locked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double unlock", ldv_set_contains(LDV_MUTEXES_i_mutex_of_inode, lock));
	/* NOTE Unlock mutex "i_mutex_of_inode" */
	ldv_set_remove(LDV_MUTEXES_i_mutex_of_inode, lock);
}
ldv_set LDV_MUTEXES_lock;

/* MODEL_FUNC Check that mutex "lock" was not locked and lock it */
void ldv_mutex_lock_lock(struct mutex *lock)
{
	/* ASSERT Mutex "lock" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_lock, lock));
	/* NOTE Lock mutex "lock" */
	ldv_set_add(LDV_MUTEXES_lock, lock);
}

/* MODEL_FUNC Check that mutex "lock" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_lock(struct mutex *lock)
{
	/* ASSERT Mutex "lock" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_lock, lock));
	/* NOTE Nondeterministically lock mutex "lock" */
	if (ldv_undef_int()) {
		/* NOTE Lock mutex "lock" */
		ldv_set_add(LDV_MUTEXES_lock, lock);
		/* NOTE Successfully locked mutex "lock" */
		return 0;
	}
	else {
		/* NOTE Could not lock mutex "lock" */
		return -EINTR;
	}
}

/* MODEL_FUNC Say whether mutex "lock" was locked in this or another thread  */
int ldv_mutex_is_locked_lock(struct mutex *lock)
{
	/* NOTE Whether mutex "lock" was locked in this thread */
	if (ldv_set_contains(LDV_MUTEXES_lock, lock)) {
		/* NOTE Mutex "lock" was locked in this thread */
		return 1;
	}
	/* NOTE Nondeterministically decide whether mutex "lock" was locked in another thread */
	else if (ldv_undef_int()) {
		/* NOTE Mutex "lock" was locked in another thread */
		return 1;
	}
	else {
		/* NOTE Mutex "lock" was not acquired in this or another thread */
		return 0;
	}
}

/* MODEL_FUNC Lock mutex "lock" if it was not locked before */
int ldv_mutex_trylock_lock(struct mutex *lock)
{
	/* ASSERT Mutex "lock" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock try", !ldv_set_contains(LDV_MUTEXES_lock, lock));

	/* NOTE Whether mutex "lock" was locked */
	if (ldv_mutex_is_locked_lock(lock)) {
		/* NOTE Mutex "lock" was locked */
		return 0;
	}
	else {
		/* NOTE Lock mutex "lock" */
		ldv_set_add(LDV_MUTEXES_lock, lock);
		/* NOTE Successfully locked mutex "lock" */
		return 1;
	}
}

/* MODEL_FUNC Decrease counter by one and if it becomes zero check that mutex "lock" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_lock(atomic_t *cnt, struct mutex *lock)
{
	/* NOTE Decrease counter by one */
	cnt->counter--;

	/* NOTE Whether counter becomes zero */
	if (cnt->counter) {
		/* NOTE Do not lock mutex "lock" since counter is greater than zero */
		return 0;
	}
	else {
		ldv_mutex_lock_lock(lock);
		/* NOTE Successfully locked mutex "lock" */
		return 1;
	}
}

/* MODEL_FUNC Check that mutex "lock" was locked and unlock it */
void ldv_mutex_unlock_lock(struct mutex *lock)
{
	/* ASSERT Mutex "lock" must be locked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double unlock", ldv_set_contains(LDV_MUTEXES_lock, lock));
	/* NOTE Unlock mutex "lock" */
	ldv_set_remove(LDV_MUTEXES_lock, lock);
}
ldv_set LDV_MUTEXES_lock_of_fb_info;

/* MODEL_FUNC Check that mutex "lock_of_fb_info" was not locked and lock it */
void ldv_mutex_lock_lock_of_fb_info(struct mutex *lock)
{
	/* ASSERT Mutex "lock_of_fb_info" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_lock_of_fb_info, lock));
	/* NOTE Lock mutex "lock_of_fb_info" */
	ldv_set_add(LDV_MUTEXES_lock_of_fb_info, lock);
}

/* MODEL_FUNC Check that mutex "lock_of_fb_info" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_lock_of_fb_info(struct mutex *lock)
{
	/* ASSERT Mutex "lock_of_fb_info" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_lock_of_fb_info, lock));
	/* NOTE Nondeterministically lock mutex "lock_of_fb_info" */
	if (ldv_undef_int()) {
		/* NOTE Lock mutex "lock_of_fb_info" */
		ldv_set_add(LDV_MUTEXES_lock_of_fb_info, lock);
		/* NOTE Successfully locked mutex "lock_of_fb_info" */
		return 0;
	}
	else {
		/* NOTE Could not lock mutex "lock_of_fb_info" */
		return -EINTR;
	}
}

/* MODEL_FUNC Say whether mutex "lock_of_fb_info" was locked in this or another thread  */
int ldv_mutex_is_locked_lock_of_fb_info(struct mutex *lock)
{
	/* NOTE Whether mutex "lock_of_fb_info" was locked in this thread */
	if (ldv_set_contains(LDV_MUTEXES_lock_of_fb_info, lock)) {
		/* NOTE Mutex "lock_of_fb_info" was locked in this thread */
		return 1;
	}
	/* NOTE Nondeterministically decide whether mutex "lock_of_fb_info" was locked in another thread */
	else if (ldv_undef_int()) {
		/* NOTE Mutex "lock_of_fb_info" was locked in another thread */
		return 1;
	}
	else {
		/* NOTE Mutex "lock_of_fb_info" was not acquired in this or another thread */
		return 0;
	}
}

/* MODEL_FUNC Lock mutex "lock_of_fb_info" if it was not locked before */
int ldv_mutex_trylock_lock_of_fb_info(struct mutex *lock)
{
	/* ASSERT Mutex "lock_of_fb_info" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock try", !ldv_set_contains(LDV_MUTEXES_lock_of_fb_info, lock));

	/* NOTE Whether mutex "lock_of_fb_info" was locked */
	if (ldv_mutex_is_locked_lock_of_fb_info(lock)) {
		/* NOTE Mutex "lock_of_fb_info" was locked */
		return 0;
	}
	else {
		/* NOTE Lock mutex "lock_of_fb_info" */
		ldv_set_add(LDV_MUTEXES_lock_of_fb_info, lock);
		/* NOTE Successfully locked mutex "lock_of_fb_info" */
		return 1;
	}
}

/* MODEL_FUNC Decrease counter by one and if it becomes zero check that mutex "lock_of_fb_info" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_lock_of_fb_info(atomic_t *cnt, struct mutex *lock)
{
	/* NOTE Decrease counter by one */
	cnt->counter--;

	/* NOTE Whether counter becomes zero */
	if (cnt->counter) {
		/* NOTE Do not lock mutex "lock_of_fb_info" since counter is greater than zero */
		return 0;
	}
	else {
		ldv_mutex_lock_lock_of_fb_info(lock);
		/* NOTE Successfully locked mutex "lock_of_fb_info" */
		return 1;
	}
}

/* MODEL_FUNC Check that mutex "lock_of_fb_info" was locked and unlock it */
void ldv_mutex_unlock_lock_of_fb_info(struct mutex *lock)
{
	/* ASSERT Mutex "lock_of_fb_info" must be locked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double unlock", ldv_set_contains(LDV_MUTEXES_lock_of_fb_info, lock));
	/* NOTE Unlock mutex "lock_of_fb_info" */
	ldv_set_remove(LDV_MUTEXES_lock_of_fb_info, lock);
}
ldv_set LDV_MUTEXES_mm_lock_of_fb_info;

/* MODEL_FUNC Check that mutex "mm_lock_of_fb_info" was not locked and lock it */
void ldv_mutex_lock_mm_lock_of_fb_info(struct mutex *lock)
{
	/* ASSERT Mutex "mm_lock_of_fb_info" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_mm_lock_of_fb_info, lock));
	/* NOTE Lock mutex "mm_lock_of_fb_info" */
	ldv_set_add(LDV_MUTEXES_mm_lock_of_fb_info, lock);
}

/* MODEL_FUNC Check that mutex "mm_lock_of_fb_info" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_mm_lock_of_fb_info(struct mutex *lock)
{
	/* ASSERT Mutex "mm_lock_of_fb_info" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_mm_lock_of_fb_info, lock));
	/* NOTE Nondeterministically lock mutex "mm_lock_of_fb_info" */
	if (ldv_undef_int()) {
		/* NOTE Lock mutex "mm_lock_of_fb_info" */
		ldv_set_add(LDV_MUTEXES_mm_lock_of_fb_info, lock);
		/* NOTE Successfully locked mutex "mm_lock_of_fb_info" */
		return 0;
	}
	else {
		/* NOTE Could not lock mutex "mm_lock_of_fb_info" */
		return -EINTR;
	}
}

/* MODEL_FUNC Say whether mutex "mm_lock_of_fb_info" was locked in this or another thread  */
int ldv_mutex_is_locked_mm_lock_of_fb_info(struct mutex *lock)
{
	/* NOTE Whether mutex "mm_lock_of_fb_info" was locked in this thread */
	if (ldv_set_contains(LDV_MUTEXES_mm_lock_of_fb_info, lock)) {
		/* NOTE Mutex "mm_lock_of_fb_info" was locked in this thread */
		return 1;
	}
	/* NOTE Nondeterministically decide whether mutex "mm_lock_of_fb_info" was locked in another thread */
	else if (ldv_undef_int()) {
		/* NOTE Mutex "mm_lock_of_fb_info" was locked in another thread */
		return 1;
	}
	else {
		/* NOTE Mutex "mm_lock_of_fb_info" was not acquired in this or another thread */
		return 0;
	}
}

/* MODEL_FUNC Lock mutex "mm_lock_of_fb_info" if it was not locked before */
int ldv_mutex_trylock_mm_lock_of_fb_info(struct mutex *lock)
{
	/* ASSERT Mutex "mm_lock_of_fb_info" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock try", !ldv_set_contains(LDV_MUTEXES_mm_lock_of_fb_info, lock));

	/* NOTE Whether mutex "mm_lock_of_fb_info" was locked */
	if (ldv_mutex_is_locked_mm_lock_of_fb_info(lock)) {
		/* NOTE Mutex "mm_lock_of_fb_info" was locked */
		return 0;
	}
	else {
		/* NOTE Lock mutex "mm_lock_of_fb_info" */
		ldv_set_add(LDV_MUTEXES_mm_lock_of_fb_info, lock);
		/* NOTE Successfully locked mutex "mm_lock_of_fb_info" */
		return 1;
	}
}

/* MODEL_FUNC Decrease counter by one and if it becomes zero check that mutex "mm_lock_of_fb_info" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_mm_lock_of_fb_info(atomic_t *cnt, struct mutex *lock)
{
	/* NOTE Decrease counter by one */
	cnt->counter--;

	/* NOTE Whether counter becomes zero */
	if (cnt->counter) {
		/* NOTE Do not lock mutex "mm_lock_of_fb_info" since counter is greater than zero */
		return 0;
	}
	else {
		ldv_mutex_lock_mm_lock_of_fb_info(lock);
		/* NOTE Successfully locked mutex "mm_lock_of_fb_info" */
		return 1;
	}
}

/* MODEL_FUNC Check that mutex "mm_lock_of_fb_info" was locked and unlock it */
void ldv_mutex_unlock_mm_lock_of_fb_info(struct mutex *lock)
{
	/* ASSERT Mutex "mm_lock_of_fb_info" must be locked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double unlock", ldv_set_contains(LDV_MUTEXES_mm_lock_of_fb_info, lock));
	/* NOTE Unlock mutex "mm_lock_of_fb_info" */
	ldv_set_remove(LDV_MUTEXES_mm_lock_of_fb_info, lock);
}
ldv_set LDV_MUTEXES_mutex_of_device;

/* MODEL_FUNC Check that mutex "mutex_of_device" was not locked and lock it */
void ldv_mutex_lock_mutex_of_device(struct mutex *lock)
{
	/* ASSERT Mutex "mutex_of_device" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_mutex_of_device, lock));
	/* NOTE Lock mutex "mutex_of_device" */
	ldv_set_add(LDV_MUTEXES_mutex_of_device, lock);
}

/* MODEL_FUNC Check that mutex "mutex_of_device" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_mutex_of_device(struct mutex *lock)
{
	/* ASSERT Mutex "mutex_of_device" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_mutex_of_device, lock));
	/* NOTE Nondeterministically lock mutex "mutex_of_device" */
	if (ldv_undef_int()) {
		/* NOTE Lock mutex "mutex_of_device" */
		ldv_set_add(LDV_MUTEXES_mutex_of_device, lock);
		/* NOTE Successfully locked mutex "mutex_of_device" */
		return 0;
	}
	else {
		/* NOTE Could not lock mutex "mutex_of_device" */
		return -EINTR;
	}
}

/* MODEL_FUNC Say whether mutex "mutex_of_device" was locked in this or another thread  */
int ldv_mutex_is_locked_mutex_of_device(struct mutex *lock)
{
	/* NOTE Whether mutex "mutex_of_device" was locked in this thread */
	if (ldv_set_contains(LDV_MUTEXES_mutex_of_device, lock)) {
		/* NOTE Mutex "mutex_of_device" was locked in this thread */
		return 1;
	}
	/* NOTE Nondeterministically decide whether mutex "mutex_of_device" was locked in another thread */
	else if (ldv_undef_int()) {
		/* NOTE Mutex "mutex_of_device" was locked in another thread */
		return 1;
	}
	else {
		/* NOTE Mutex "mutex_of_device" was not acquired in this or another thread */
		return 0;
	}
}

/* MODEL_FUNC Lock mutex "mutex_of_device" if it was not locked before */
int ldv_mutex_trylock_mutex_of_device(struct mutex *lock)
{
	/* ASSERT Mutex "mutex_of_device" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock try", !ldv_set_contains(LDV_MUTEXES_mutex_of_device, lock));

	/* NOTE Whether mutex "mutex_of_device" was locked */
	if (ldv_mutex_is_locked_mutex_of_device(lock)) {
		/* NOTE Mutex "mutex_of_device" was locked */
		return 0;
	}
	else {
		/* NOTE Lock mutex "mutex_of_device" */
		ldv_set_add(LDV_MUTEXES_mutex_of_device, lock);
		/* NOTE Successfully locked mutex "mutex_of_device" */
		return 1;
	}
}

/* MODEL_FUNC Decrease counter by one and if it becomes zero check that mutex "mutex_of_device" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_mutex_of_device(atomic_t *cnt, struct mutex *lock)
{
	/* NOTE Decrease counter by one */
	cnt->counter--;

	/* NOTE Whether counter becomes zero */
	if (cnt->counter) {
		/* NOTE Do not lock mutex "mutex_of_device" since counter is greater than zero */
		return 0;
	}
	else {
		ldv_mutex_lock_mutex_of_device(lock);
		/* NOTE Successfully locked mutex "mutex_of_device" */
		return 1;
	}
}

/* MODEL_FUNC Check that mutex "mutex_of_device" was locked and unlock it */
void ldv_mutex_unlock_mutex_of_device(struct mutex *lock)
{
	/* ASSERT Mutex "mutex_of_device" must be locked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double unlock", ldv_set_contains(LDV_MUTEXES_mutex_of_device, lock));
	/* NOTE Unlock mutex "mutex_of_device" */
	ldv_set_remove(LDV_MUTEXES_mutex_of_device, lock);
}
ldv_set LDV_MUTEXES_registration_lock;

/* MODEL_FUNC Check that mutex "registration_lock" was not locked and lock it */
void ldv_mutex_lock_registration_lock(struct mutex *lock)
{
	/* ASSERT Mutex "registration_lock" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_registration_lock, lock));
	/* NOTE Lock mutex "registration_lock" */
	ldv_set_add(LDV_MUTEXES_registration_lock, lock);
}

/* MODEL_FUNC Check that mutex "registration_lock" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_registration_lock(struct mutex *lock)
{
	/* ASSERT Mutex "registration_lock" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_registration_lock, lock));
	/* NOTE Nondeterministically lock mutex "registration_lock" */
	if (ldv_undef_int()) {
		/* NOTE Lock mutex "registration_lock" */
		ldv_set_add(LDV_MUTEXES_registration_lock, lock);
		/* NOTE Successfully locked mutex "registration_lock" */
		return 0;
	}
	else {
		/* NOTE Could not lock mutex "registration_lock" */
		return -EINTR;
	}
}

/* MODEL_FUNC Say whether mutex "registration_lock" was locked in this or another thread  */
int ldv_mutex_is_locked_registration_lock(struct mutex *lock)
{
	/* NOTE Whether mutex "registration_lock" was locked in this thread */
	if (ldv_set_contains(LDV_MUTEXES_registration_lock, lock)) {
		/* NOTE Mutex "registration_lock" was locked in this thread */
		return 1;
	}
	/* NOTE Nondeterministically decide whether mutex "registration_lock" was locked in another thread */
	else if (ldv_undef_int()) {
		/* NOTE Mutex "registration_lock" was locked in another thread */
		return 1;
	}
	else {
		/* NOTE Mutex "registration_lock" was not acquired in this or another thread */
		return 0;
	}
}

/* MODEL_FUNC Lock mutex "registration_lock" if it was not locked before */
int ldv_mutex_trylock_registration_lock(struct mutex *lock)
{
	/* ASSERT Mutex "registration_lock" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock try", !ldv_set_contains(LDV_MUTEXES_registration_lock, lock));

	/* NOTE Whether mutex "registration_lock" was locked */
	if (ldv_mutex_is_locked_registration_lock(lock)) {
		/* NOTE Mutex "registration_lock" was locked */
		return 0;
	}
	else {
		/* NOTE Lock mutex "registration_lock" */
		ldv_set_add(LDV_MUTEXES_registration_lock, lock);
		/* NOTE Successfully locked mutex "registration_lock" */
		return 1;
	}
}

/* MODEL_FUNC Decrease counter by one and if it becomes zero check that mutex "registration_lock" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_registration_lock(atomic_t *cnt, struct mutex *lock)
{
	/* NOTE Decrease counter by one */
	cnt->counter--;

	/* NOTE Whether counter becomes zero */
	if (cnt->counter) {
		/* NOTE Do not lock mutex "registration_lock" since counter is greater than zero */
		return 0;
	}
	else {
		ldv_mutex_lock_registration_lock(lock);
		/* NOTE Successfully locked mutex "registration_lock" */
		return 1;
	}
}

/* MODEL_FUNC Check that mutex "registration_lock" was locked and unlock it */
void ldv_mutex_unlock_registration_lock(struct mutex *lock)
{
	/* ASSERT Mutex "registration_lock" must be locked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double unlock", ldv_set_contains(LDV_MUTEXES_registration_lock, lock));
	/* NOTE Unlock mutex "registration_lock" */
	ldv_set_remove(LDV_MUTEXES_registration_lock, lock);
}
ldv_set LDV_MUTEXES_update_lock_of_backlight_device;

/* MODEL_FUNC Check that mutex "update_lock_of_backlight_device" was not locked and lock it */
void ldv_mutex_lock_update_lock_of_backlight_device(struct mutex *lock)
{
	/* ASSERT Mutex "update_lock_of_backlight_device" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_update_lock_of_backlight_device, lock));
	/* NOTE Lock mutex "update_lock_of_backlight_device" */
	ldv_set_add(LDV_MUTEXES_update_lock_of_backlight_device, lock);
}

/* MODEL_FUNC Check that mutex "update_lock_of_backlight_device" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_update_lock_of_backlight_device(struct mutex *lock)
{
	/* ASSERT Mutex "update_lock_of_backlight_device" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock", !ldv_set_contains(LDV_MUTEXES_update_lock_of_backlight_device, lock));
	/* NOTE Nondeterministically lock mutex "update_lock_of_backlight_device" */
	if (ldv_undef_int()) {
		/* NOTE Lock mutex "update_lock_of_backlight_device" */
		ldv_set_add(LDV_MUTEXES_update_lock_of_backlight_device, lock);
		/* NOTE Successfully locked mutex "update_lock_of_backlight_device" */
		return 0;
	}
	else {
		/* NOTE Could not lock mutex "update_lock_of_backlight_device" */
		return -EINTR;
	}
}

/* MODEL_FUNC Say whether mutex "update_lock_of_backlight_device" was locked in this or another thread  */
int ldv_mutex_is_locked_update_lock_of_backlight_device(struct mutex *lock)
{
	/* NOTE Whether mutex "update_lock_of_backlight_device" was locked in this thread */
	if (ldv_set_contains(LDV_MUTEXES_update_lock_of_backlight_device, lock)) {
		/* NOTE Mutex "update_lock_of_backlight_device" was locked in this thread */
		return 1;
	}
	/* NOTE Nondeterministically decide whether mutex "update_lock_of_backlight_device" was locked in another thread */
	else if (ldv_undef_int()) {
		/* NOTE Mutex "update_lock_of_backlight_device" was locked in another thread */
		return 1;
	}
	else {
		/* NOTE Mutex "update_lock_of_backlight_device" was not acquired in this or another thread */
		return 0;
	}
}

/* MODEL_FUNC Lock mutex "update_lock_of_backlight_device" if it was not locked before */
int ldv_mutex_trylock_update_lock_of_backlight_device(struct mutex *lock)
{
	/* ASSERT Mutex "update_lock_of_backlight_device" must be unlocked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double lock try", !ldv_set_contains(LDV_MUTEXES_update_lock_of_backlight_device, lock));

	/* NOTE Whether mutex "update_lock_of_backlight_device" was locked */
	if (ldv_mutex_is_locked_update_lock_of_backlight_device(lock)) {
		/* NOTE Mutex "update_lock_of_backlight_device" was locked */
		return 0;
	}
	else {
		/* NOTE Lock mutex "update_lock_of_backlight_device" */
		ldv_set_add(LDV_MUTEXES_update_lock_of_backlight_device, lock);
		/* NOTE Successfully locked mutex "update_lock_of_backlight_device" */
		return 1;
	}
}

/* MODEL_FUNC Decrease counter by one and if it becomes zero check that mutex "update_lock_of_backlight_device" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_update_lock_of_backlight_device(atomic_t *cnt, struct mutex *lock)
{
	/* NOTE Decrease counter by one */
	cnt->counter--;

	/* NOTE Whether counter becomes zero */
	if (cnt->counter) {
		/* NOTE Do not lock mutex "update_lock_of_backlight_device" since counter is greater than zero */
		return 0;
	}
	else {
		ldv_mutex_lock_update_lock_of_backlight_device(lock);
		/* NOTE Successfully locked mutex "update_lock_of_backlight_device" */
		return 1;
	}
}

/* MODEL_FUNC Check that mutex "update_lock_of_backlight_device" was locked and unlock it */
void ldv_mutex_unlock_update_lock_of_backlight_device(struct mutex *lock)
{
	/* ASSERT Mutex "update_lock_of_backlight_device" must be locked */
	ldv_assert("linux:kernel:locking:mutex::one thread:double unlock", ldv_set_contains(LDV_MUTEXES_update_lock_of_backlight_device, lock));
	/* NOTE Unlock mutex "update_lock_of_backlight_device" */
	ldv_set_remove(LDV_MUTEXES_update_lock_of_backlight_device, lock);
}
/* MODEL_FUNC Make all mutexes unlocked at the beginning */
void ldv_initialize(void)
{
	/* NOTE Mutex "bl_curve_mutex_of_fb_info" is unlocked at the beginning */
	ldv_set_init(LDV_MUTEXES_bl_curve_mutex_of_fb_info);
	/* NOTE Mutex "i_mutex_of_inode" is unlocked at the beginning */
	ldv_set_init(LDV_MUTEXES_i_mutex_of_inode);
	/* NOTE Mutex "lock" is unlocked at the beginning */
	ldv_set_init(LDV_MUTEXES_lock);
	/* NOTE Mutex "lock_of_fb_info" is unlocked at the beginning */
	ldv_set_init(LDV_MUTEXES_lock_of_fb_info);
	/* NOTE Mutex "mm_lock_of_fb_info" is unlocked at the beginning */
	ldv_set_init(LDV_MUTEXES_mm_lock_of_fb_info);
	/* NOTE Mutex "mutex_of_device" is unlocked at the beginning */
	ldv_set_init(LDV_MUTEXES_mutex_of_device);
	/* NOTE Mutex "registration_lock" is unlocked at the beginning */
	ldv_set_init(LDV_MUTEXES_registration_lock);
	/* NOTE Mutex "update_lock_of_backlight_device" is unlocked at the beginning */
	ldv_set_init(LDV_MUTEXES_update_lock_of_backlight_device);
}

/* MODEL_FUNC Check that all mutexes are unlocked at the end */
void ldv_check_final_state(void)
{
	/* ASSERT Mutex "bl_curve_mutex_of_fb_info" must be unlocked at the end */
	ldv_assert("linux:kernel:locking:mutex::one thread:locked at exit", ldv_set_is_empty(LDV_MUTEXES_bl_curve_mutex_of_fb_info));
	/* ASSERT Mutex "i_mutex_of_inode" must be unlocked at the end */
	ldv_assert("linux:kernel:locking:mutex::one thread:locked at exit", ldv_set_is_empty(LDV_MUTEXES_i_mutex_of_inode));
	/* ASSERT Mutex "lock" must be unlocked at the end */
	ldv_assert("linux:kernel:locking:mutex::one thread:locked at exit", ldv_set_is_empty(LDV_MUTEXES_lock));
	/* ASSERT Mutex "lock_of_fb_info" must be unlocked at the end */
	ldv_assert("linux:kernel:locking:mutex::one thread:locked at exit", ldv_set_is_empty(LDV_MUTEXES_lock_of_fb_info));
	/* ASSERT Mutex "mm_lock_of_fb_info" must be unlocked at the end */
	ldv_assert("linux:kernel:locking:mutex::one thread:locked at exit", ldv_set_is_empty(LDV_MUTEXES_mm_lock_of_fb_info));
	/* ASSERT Mutex "mutex_of_device" must be unlocked at the end */
	ldv_assert("linux:kernel:locking:mutex::one thread:locked at exit", ldv_set_is_empty(LDV_MUTEXES_mutex_of_device));
	/* ASSERT Mutex "registration_lock" must be unlocked at the end */
	ldv_assert("linux:kernel:locking:mutex::one thread:locked at exit", ldv_set_is_empty(LDV_MUTEXES_registration_lock));
	/* ASSERT Mutex "update_lock_of_backlight_device" must be unlocked at the end */
	ldv_assert("linux:kernel:locking:mutex::one thread:locked at exit", ldv_set_is_empty(LDV_MUTEXES_update_lock_of_backlight_device));
}